DOCUMENTATION_WEBSITE = 'https://flo-ai.rootflo.ai'
DOCUMENTATION_ROUTER_ANCHOR = f'{DOCUMENTATION_WEBSITE}/basics/routers'
DOCUMENTATION_AGENT_ANCHOR = f'{DOCUMENTATION_WEBSITE}/basics/agents'
DOCUMENTATION_AGENT_TOOLS = f'{DOCUMENTATION_WEBSITE}/basics/tools'
